@@ -1137,6 +1137,41 @@ gst_element_class_set_details (GstElementClass * klass,
   __gst_element_details_copy (&klass->details, details);
 }
 
+/**
+ * gst_element_class_set_details_simple:
+ * @klass: class to set details for
+ * @longname: The long English name of the element. E.g. "File Sink"
+ * @classification: String describing the type of element, as an unordered list
+ * separated with slashes ('/'). See draft-klass.txt of the design docs
+ * for more details and common types. E.g: "Sink/File"
+ * @description: Sentence describing the purpose of the element. 
+ * E.g: "Write stream to a file"
+ * @author: Name and contact details of the author(s). Use \n to separate 
+ * multiple author details. E.g: "Joe Bloggs &lt;joe.blogs at foo.com&gt;"
+ *
+ * Sets the detailed information for a #GstElementClass. Simpler version of 
+ * gst_element_class_set_details() that generates less linker overhead.
+ * <note>This function is for use in _base_init functions only.</note>
+ *
+ * The detail parameter strings are copied into the #GstElementDetails for
+ * the element class.
+ *
+ * Since: 0.10.14
+ */
+void
+gst_element_class_set_details_simple (GstElementClass * klass,
+    const gchar * longname, const gchar * classification,
+    const gchar * description, const gchar * author)
+{
+  const GstElementDetails details =
+      GST_ELEMENT_DETAILS ((gchar *) longname, (gchar *) classification,
+      (gchar *) description, (gchar *) author);
+
+  g_return_if_fail (GST_IS_ELEMENT_CLASS (klass));
+
+  __gst_element_details_copy (&klass->details, &details);
+}
+
 /**
  * gst_element_class_get_pad_template_list:
  * @element_class: a #GstElementClass to get pad templates of.
@@ -1858,11 +1893,11 @@ gst_element_get_state_func (GstElement * element,
         ret = GST_STATE_CHANGE_FAILURE;
       }
     }
-  }
-  /* if nothing is pending anymore we can return SUCCESS */
-  if (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING) {
-    GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "nothing pending");
-    ret = GST_STATE_CHANGE_SUCCESS;
+    /* if nothing is pending anymore we can return SUCCESS */
+    if (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING) {
+      GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "nothing pending");
+      ret = GST_STATE_CHANGE_SUCCESS;
+    }
   }
 
 done:
@@ -1887,7 +1922,7 @@ interrupted:
     if (pending)
       *pending = GST_STATE_VOID_PENDING;
 
-    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element, "get_state() interruped");
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element, "interruped");
 
     GST_OBJECT_UNLOCK (element);
 
@@ -2100,8 +2135,13 @@ complete:
      * We do signal the cond though as a _get_state() might be blocking
      * on it. */
     if (old_state != old_next || old_ret == GST_STATE_CHANGE_ASYNC) {
-      message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
-          old_state, old_next, GST_STATE_VOID_PENDING);
+      GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
+          "posting state-changed %s to %s",
+          gst_element_state_get_name (old_state),
+          gst_element_state_get_name (old_next));
+      message =
+          gst_message_new_state_changed (GST_OBJECT_CAST (element), old_state,
+          old_next, GST_STATE_VOID_PENDING);
       gst_element_post_message (element, message);
     }
 
@@ -2891,7 +2931,8 @@ gst_element_set_bus (GstElement * element, GstBus * bus)
  * gst_element_get_bus:
  * @element: a #GstElement to get the bus of.
  *
- * Returns the bus of the element.
+ * Returns the bus of the element. Note that only a #GstPipeline will provide a
+ * bus for the application.
  *
  * Returns: the element's #GstBus. unref after usage.
  *
