@@ -48,7 +48,7 @@ G_BEGIN_DECLS
 typedef enum {
   GST_EVENT_TYPE_UPSTREAM	= 1 << 0,
   GST_EVENT_TYPE_DOWNSTREAM	= 1 << 1,
-  GST_EVENT_TYPE_SERIALIZED	= 1 << 2,
+  GST_EVENT_TYPE_SERIALIZED	= 1 << 2
 } GstEventTypeFlags;
 
 /**
@@ -91,6 +91,7 @@ typedef enum {
  * @GST_EVENT_NAVIGATION: Navigation events are usually used for communicating
  *                        user requests, such as mouse or keyboard movements,
  *                        to upstream elements.
+ * @GST_EVENT_LATENCY: Notification of new latency adjustment. Since: 0.10.12
  * @GST_EVENT_CUSTOM_UPSTREAM: Upstream custom event
  * @GST_EVENT_CUSTOM_DOWNSTREAM: Downstream custom event that travels in the
  *                        data flow.
@@ -109,26 +110,27 @@ typedef enum {
  */
 /* NOTE: keep in sync with quark registration in gstevent.c */
 typedef enum {
-  GST_EVENT_UNKNOWN		= GST_EVENT_MAKE_TYPE (0, 0),
+  GST_EVENT_UNKNOWN		  = GST_EVENT_MAKE_TYPE (0, 0),
   /* bidirectional events */
-  GST_EVENT_FLUSH_START		= GST_EVENT_MAKE_TYPE (1, FLAG(BOTH)),
-  GST_EVENT_FLUSH_STOP		= GST_EVENT_MAKE_TYPE (2, FLAG(BOTH) | FLAG(SERIALIZED)),
+  GST_EVENT_FLUSH_START		  = GST_EVENT_MAKE_TYPE (1, FLAG(BOTH)),
+  GST_EVENT_FLUSH_STOP		  = GST_EVENT_MAKE_TYPE (2, FLAG(BOTH) | FLAG(SERIALIZED)),
   /* downstream serialized events */
-  GST_EVENT_EOS			= GST_EVENT_MAKE_TYPE (5, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
-  GST_EVENT_NEWSEGMENT		= GST_EVENT_MAKE_TYPE (6, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
-  GST_EVENT_TAG			= GST_EVENT_MAKE_TYPE (7, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
-  GST_EVENT_BUFFERSIZE		= GST_EVENT_MAKE_TYPE (8, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
+  GST_EVENT_EOS			  = GST_EVENT_MAKE_TYPE (5, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
+  GST_EVENT_NEWSEGMENT		  = GST_EVENT_MAKE_TYPE (6, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
+  GST_EVENT_TAG			  = GST_EVENT_MAKE_TYPE (7, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
+  GST_EVENT_BUFFERSIZE		  = GST_EVENT_MAKE_TYPE (8, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
   /* upstream events */
-  GST_EVENT_QOS			= GST_EVENT_MAKE_TYPE (15, FLAG(UPSTREAM)),
-  GST_EVENT_SEEK		= GST_EVENT_MAKE_TYPE (16, FLAG(UPSTREAM)),
-  GST_EVENT_NAVIGATION		= GST_EVENT_MAKE_TYPE (17, FLAG(UPSTREAM)),
+  GST_EVENT_QOS			  = GST_EVENT_MAKE_TYPE (15, FLAG(UPSTREAM)),
+  GST_EVENT_SEEK		  = GST_EVENT_MAKE_TYPE (16, FLAG(UPSTREAM)),
+  GST_EVENT_NAVIGATION		  = GST_EVENT_MAKE_TYPE (17, FLAG(UPSTREAM)),
+  GST_EVENT_LATENCY		  = GST_EVENT_MAKE_TYPE (18, FLAG(UPSTREAM)),
 
   /* custom events start here */
-  GST_EVENT_CUSTOM_UPSTREAM	= GST_EVENT_MAKE_TYPE (32, FLAG(UPSTREAM)),
-  GST_EVENT_CUSTOM_DOWNSTREAM	= GST_EVENT_MAKE_TYPE (32, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
+  GST_EVENT_CUSTOM_UPSTREAM	  = GST_EVENT_MAKE_TYPE (32, FLAG(UPSTREAM)),
+  GST_EVENT_CUSTOM_DOWNSTREAM	  = GST_EVENT_MAKE_TYPE (32, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
   GST_EVENT_CUSTOM_DOWNSTREAM_OOB = GST_EVENT_MAKE_TYPE (32, FLAG(DOWNSTREAM)),
-  GST_EVENT_CUSTOM_BOTH		= GST_EVENT_MAKE_TYPE (32, FLAG(BOTH) | FLAG(SERIALIZED)),
-  GST_EVENT_CUSTOM_BOTH_OOB	= GST_EVENT_MAKE_TYPE (32, FLAG(BOTH))
+  GST_EVENT_CUSTOM_BOTH		  = GST_EVENT_MAKE_TYPE (32, FLAG(BOTH) | FLAG(SERIALIZED)),
+  GST_EVENT_CUSTOM_BOTH_OOB	  = GST_EVENT_MAKE_TYPE (32, FLAG(BOTH))
 } GstEventType;
 #undef FLAG
 
@@ -230,10 +232,10 @@ typedef struct _GstEventClass GstEventClass;
  * @GST_SEEK_TYPE_SET: absolute position is requested
  * @GST_SEEK_TYPE_END: relative position to duration is requested
  *
- * The different types of seek events. When constructing a seek event, a format,
- * a seek method and optional flags are to be provided. The seek event is then
- * inserted into the graph with gst_pad_send_event() or
- * gst_element_send_event().
+ * The different types of seek events. When constructing a seek event with
+ * gst_event_new_seek(), a format, a seek method and optional flags are to
+ * be provided. The seek event is then inserted into the graph with
+ * gst_pad_send_event() or gst_element_send_event().
  */
 typedef enum {
   /* one of these */
@@ -253,7 +255,8 @@ typedef enum {
  *		       faster but less accurate.
  * @GST_SEEK_FLAG_SEGMENT: perform a segment seek.
  *
- * Flags to be used with gst_element_seek() or gst_event_new_seek()
+ * Flags to be used with gst_element_seek() or gst_event_new_seek(). All flags
+ * can be used together.
  *
  * A non flushing seek might take some time to perform as the currently
  * playing data in the pipeline will not be cleared.
@@ -264,10 +267,10 @@ typedef enum {
  *
  * When performing a segment seek: after the playback of the segment completes,
  * no EOS will be emmited by the element that performed the seek, but a
- * SEGMENT_DONE message will be posted on the bus by the element. When this
- * message is posted, it is possible to send a new seek event to continue
- * playback. With this seek method it is possible to perform seemless looping
- * or simple linear editing.
+ * #GST_MESSAGE_SEGMENT_DONE message will be posted on the bus by the element.
+ * When this message is posted, it is possible to send a new seek event to
+ * continue playback. With this seek method it is possible to perform seemless
+ * looping or simple linear editing.
  */
 typedef enum {
   GST_SEEK_FLAG_NONE		= 0,
@@ -322,12 +325,14 @@ GType		gst_event_get_type		(void);
 /* refcounting */
 /**
  * gst_event_ref:
- * @ev: The event to refcount
+ * @event: The event to refcount
  *
  * Increase the refcount of this event.
+ *
+ * Returns: @event (for convenience when doing assignments)
  */
 #ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC GstEvent * gst_event_ref (GstEvent * ev);
+G_INLINE_FUNC GstEvent * gst_event_ref (GstEvent * event);
 #endif
 
 static inline GstEvent *
@@ -399,7 +404,7 @@ void		gst_event_parse_tag		(GstEvent *event, GstTagList **taglist);
 /* buffer */
 GstEvent *	gst_event_new_buffer_size	(GstFormat format, gint64 minsize, gint64 maxsize,
 						 gboolean async);
-void 		gst_event_parse_buffer_size	(GstEvent *event, GstFormat *format, gint64 *minsize,
+void		gst_event_parse_buffer_size	(GstEvent *event, GstFormat *format, gint64 *minsize,
 						 gint64 *maxsize, gboolean *async);
 
 /* QOS events */
@@ -409,15 +414,19 @@ void		gst_event_parse_qos		(GstEvent *event, gdouble *proportion, GstClockTimeDi
 						 GstClockTime *timestamp);
 /* seek event */
 GstEvent*	gst_event_new_seek		(gdouble rate, GstFormat format, GstSeekFlags flags,
-						 GstSeekType cur_type, gint64 cur,
+						 GstSeekType start_type, gint64 start,
 						 GstSeekType stop_type, gint64 stop);
 void		gst_event_parse_seek		(GstEvent *event, gdouble *rate, GstFormat *format,
 		                                 GstSeekFlags *flags,
-						 GstSeekType *cur_type, gint64 *cur,
+						 GstSeekType *start_type, gint64 *start,
 						 GstSeekType *stop_type, gint64 *stop);
 /* navigation event */
 GstEvent*	gst_event_new_navigation	(GstStructure *structure);
 
+/* latency event */
+GstEvent*	gst_event_new_latency		(GstClockTime latency);
+void		gst_event_parse_latency		(GstEvent *event, GstClockTime *latency);
+
 G_END_DECLS
 
 #endif /* __GST_EVENT_H__ */
