@@ -51,7 +51,7 @@
  * the variables and set the default category.
  * <informalexample>
  * <programlisting>
- * GST_DEBUG_CATEGORY (my_category);            // define category
+ * GST_DEBUG_CATEGORY_STATIC (my_category);     // define category (statically)
  * &hash;define GST_CAT_DEFAULT my_category     // set as default
  * </programlisting>
  * </informalexample>
@@ -106,6 +106,7 @@
 #include <string.h>             /* G_VA_COPY */
 #include "gst_private.h"
 #include "gstutils.h"
+#include "gstsegment.h"
 #ifdef HAVE_VALGRIND
 #  include <valgrind/valgrind.h>
 #endif
@@ -149,7 +150,9 @@ static void gst_debug_reset_threshold (gpointer category, gpointer unused);
 static void gst_debug_reset_all_thresholds (void);
 
 #ifdef HAVE_PRINTF_EXTENSION
-static int _gst_info_printf_extension (FILE * stream,
+static int _gst_info_printf_extension_ptr (FILE * stream,
+    const struct printf_info *info, const void *const *args);
+static int _gst_info_printf_extension_segment (FILE * stream,
     const struct printf_info *info, const void *const *args);
 static int _gst_info_printf_extension_arginfo (const struct printf_info *info,
     size_t n, int *argtypes);
@@ -188,8 +191,11 @@ static GSList *__log_functions = NULL;
 
 static gint __default_level;
 static gint __use_color;
-gboolean __gst_debug_enabled = TRUE;
 
+/* disabled by default, as soon as some threshold is set > NONE,
+ * it becomes enabled. */
+gboolean __gst_debug_enabled = FALSE;
+GstDebugLevel __gst_debug_min = GST_LEVEL_NONE;
 
 GstDebugCategory *GST_CAT_DEFAULT = NULL;
 
@@ -222,6 +228,7 @@ GstDebugCategory *GST_CAT_CALL_TRACE = NULL;
 GstDebugCategory *GST_CAT_SIGNAL = NULL;
 GstDebugCategory *GST_CAT_PROBE = NULL;
 GstDebugCategory *GST_CAT_REGISTRY = NULL;
+GstDebugCategory *GST_CAT_QOS = NULL;
 
 /* FIXME: export this? */
 gboolean
@@ -239,11 +246,9 @@ __gst_in_valgrind (void)
 #ifdef HAVE_VALGRIND
     if (RUNNING_ON_VALGRIND) {
       GST_CAT_INFO (GST_CAT_GST_INIT, "we're running inside valgrind");
-      VALGRIND_PRINTF
-          ("GStreamer has detected that it is running inside valgrind.");
-      VALGRIND_PRINTF
-          ("It might now take different code paths to ease debugging.");
-      VALGRIND_PRINTF ("Of course, this may also lead to different bugs.");
+      printf ("GStreamer has detected that it is running inside valgrind.\n");
+      printf ("It might now take different code paths to ease debugging.\n");
+      printf ("Of course, this may also lead to different bugs.\n");
       in_valgrind = GST_VG_INSIDE;
     } else {
       GST_CAT_LOG (GST_CAT_GST_INIT, "not doing extra valgrind stuff");
@@ -277,8 +282,10 @@ _gst_debug_init (void)
   start_time = GST_TIMEVAL_TO_TIME (current);
 
 #ifdef HAVE_PRINTF_EXTENSION
-  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension,
+  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension_ptr,
       _gst_info_printf_extension_arginfo);
+  register_printf_function (GST_SEGMENT_FORMAT[0],
+      _gst_info_printf_extension_segment, _gst_info_printf_extension_arginfo);
 #endif
 
   /* do NOT use a single debug function before this line has been run */
@@ -345,6 +352,7 @@ _gst_debug_init (void)
   GST_CAT_PROBE = _gst_debug_category_new ("GST_PROBE",
       GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, "pad probes");
   GST_CAT_REGISTRY = _gst_debug_category_new ("GST_REGISTRY", 0, "registry");
+  GST_CAT_QOS = _gst_debug_category_new ("GST_QOS", 0, "QoS");
 
 
   /* print out the valgrind message if we're in valgrind */
@@ -483,10 +491,72 @@ gst_debug_print_object (gpointer ptr)
   if (G_IS_OBJECT (object)) {
     return g_strdup_printf ("<%s@%p>", G_OBJECT_TYPE_NAME (object), object);
   }
+  if (GST_IS_MESSAGE (object)) {
+    GstMessage *msg = GST_MESSAGE_CAST (object);
+    gchar *s, *ret;
+
+    if (msg->structure) {
+      s = gst_structure_to_string (msg->structure);
+    } else {
+      s = g_strdup ("(NULL)");
+    }
+
+    ret = g_strdup_printf ("%s message from element '%s': %s",
+        GST_MESSAGE_TYPE_NAME (msg), (msg->src != NULL) ?
+        GST_ELEMENT_NAME (msg->src) : "(NULL)", s);
+    g_free (s);
+    return ret;
+  }
 
   return g_strdup_printf ("%p", ptr);
 }
 
+#ifdef HAVE_PRINTF_EXTENSION
+
+static gchar *
+gst_debug_print_segment (gpointer ptr)
+{
+  GstSegment *segment = (GstSegment *) ptr;
+
+  /* nicely printed segment */
+  if (segment == NULL) {
+    return g_strdup ("(NULL)");
+  }
+
+  switch (segment->format) {
+    case GST_FORMAT_UNDEFINED:{
+      return g_strdup_printf ("UNDEFINED segment");
+    }
+    case GST_FORMAT_TIME:{
+      return g_strdup_printf ("time segment start=%" GST_TIME_FORMAT
+          ", stop=%" GST_TIME_FORMAT ", last_stop=%" GST_TIME_FORMAT
+          ", duration=%" GST_TIME_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          GST_TIME_ARGS (segment->start), GST_TIME_ARGS (segment->stop),
+          GST_TIME_ARGS (segment->last_stop), GST_TIME_ARGS (segment->duration),
+          segment->rate, segment->applied_rate, (guint) segment->flags,
+          GST_TIME_ARGS (segment->time), GST_TIME_ARGS (segment->accum));
+    }
+    default:{
+      const gchar *format_name;
+
+      format_name = gst_format_get_name (segment->format);
+      if (G_UNLIKELY (format_name == NULL))
+        format_name = "(UNKNOWN FORMAT)";
+      return g_strdup_printf ("%s segment start=%" G_GINT64_FORMAT
+          ", stop=%" G_GINT64_FORMAT ", last_stop=%" G_GINT64_FORMAT
+          ", duration=%" G_GINT64_FORMAT ", rate=%f, applied_rate=%f"
+          ", flags=0x%02x, time=%" GST_TIME_FORMAT ", accum=%" GST_TIME_FORMAT,
+          format_name, segment->start, segment->stop, segment->last_stop,
+          segment->duration, segment->rate, segment->applied_rate,
+          (guint) segment->flags, GST_TIME_ARGS (segment->time),
+          GST_TIME_ARGS (segment->accum));
+    }
+  }
+}
+
+#endif /* HAVE_PRINTF_EXTENSION */
+
 /**
  * gst_debug_construct_term_color:
  * @colorinfo: the color info
@@ -552,11 +622,20 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar *clear;
   gchar *obj = NULL;
   gchar pidcolor[10];
+  const gchar *levelcolor;
   gint pid;
   GTimeVal now;
   GstClockTime elapsed;
   gboolean free_color = TRUE;
   gboolean free_obj = TRUE;
+  static const gchar *levelcolormap[] = {
+    "\033[37m",                 /* GST_LEVEL_NONE */
+    "\033[31;01m",              /* GST_LEVEL_ERROR */
+    "\033[33;01m",              /* GST_LEVEL_WARNING */
+    "\033[32;01m",              /* GST_LEVEL_INFO */
+    "\033[36m",                 /* GST_LEVEL_DEBUG */
+    "\033[37m"                  /* GST_LEVEL_LOG */
+  };
 
   if (level > gst_debug_category_get_threshold (category))
     return;
@@ -569,11 +648,13 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
         (category));
     clear = "\033[00m";
     g_sprintf (pidcolor, "\033[3%1dm", pid % 6 + 31);
+    levelcolor = levelcolormap[level];
   } else {
     color = "\0";
     free_color = FALSE;
     clear = "";
     pidcolor[0] = '\0';
+    levelcolor = "\0";
   }
 
   if (object) {
@@ -585,12 +666,21 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
 
   g_get_current_time (&now);
   elapsed = GST_TIMEVAL_TO_TIME (now) - start_time;
-  g_printerr ("%s (%p - %" GST_TIME_FORMAT
-      ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
-      gst_debug_level_get_name (level), g_thread_self (),
-      GST_TIME_ARGS (elapsed), color,
-      gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
-      color, file, line, function, obj, clear, gst_debug_message_get (message));
+
+  /*
+     g_printerr ("%s (%p - %" GST_TIME_FORMAT ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
+     gst_debug_level_get_name (level), g_thread_self (),
+     GST_TIME_ARGS (elapsed), color,
+     gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
+     color, file, line, function, obj, clear, gst_debug_message_get (message));
+   */
+
+  g_printerr ("%" GST_TIME_FORMAT
+      " %s%5d%s %p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
+      pidcolor, pid, clear, g_thread_self (), levelcolor,
+      gst_debug_level_get_name (level), clear, color,
+      gst_debug_category_get_name (category), file, line, function, obj, clear,
+      gst_debug_message_get (message));
 
   if (free_color)
     g_free (color);
@@ -665,7 +755,7 @@ gst_debug_add_log_function (GstLogFunction func, gpointer data)
 static gint
 gst_debug_compare_log_function_by_func (gconstpointer entry, gconstpointer func)
 {
-  gpointer entryfunc = ((LogFuncEntry *) entry)->func;
+  gpointer entryfunc = (gpointer) (((LogFuncEntry *) entry)->func);
 
   return (entryfunc < func) ? -1 : (entryfunc > func) ? 1 : 0;
 }
@@ -720,7 +810,7 @@ gst_debug_remove_log_function (GstLogFunction func)
 
   removals =
       gst_debug_remove_with_compare_func
-      (gst_debug_compare_log_function_by_func, func);
+      (gst_debug_compare_log_function_by_func, (gpointer) func);
   GST_DEBUG ("removed log function %p %d times from log function list", func,
       removals);
 
@@ -789,6 +879,10 @@ void
 gst_debug_set_active (gboolean active)
 {
   __gst_debug_enabled = active;
+  if (active)
+    __gst_debug_min = GST_LEVEL_COUNT;
+  else
+    __gst_debug_min = GST_LEVEL_NONE;
 }
 
 /**
@@ -1004,6 +1098,11 @@ gst_debug_category_set_threshold (GstDebugCategory * category,
 {
   g_return_if_fail (category != NULL);
 
+  if (level > __gst_debug_min) {
+    __gst_debug_enabled = TRUE;
+    __gst_debug_min = level;
+  }
+
   gst_atomic_int_set (&category->threshold, level);
 }
 
@@ -1086,8 +1185,6 @@ gst_debug_category_get_description (GstDebugCategory * category)
  * Returns a snapshot of a all categories that are currently in use . This list
  * may change anytime.
  * The caller has to free the list after use.
- * <emphasis>This function is not threadsafe, so only use it while only the
- * main thread is running.</emphasis>
  *
  * Returns: the list of categories
  */
@@ -1115,40 +1212,44 @@ _gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
 /* This function MUST NOT return NULL */
      const gchar *_gst_debug_nameof_funcptr (GstDebugFuncPtr func)
 {
-  gpointer ptr = (gpointer) func;
   gchar *ptrname;
 
 #ifdef HAVE_DLADDR
   Dl_info dlinfo;
 #endif
 
-  if (__gst_function_pointers) {
-    g_static_mutex_lock (&__dbg_functions_mutex);
-    ptrname = g_hash_table_lookup (__gst_function_pointers, ptr);
+  if (G_UNLIKELY (func == NULL))
+    return "(NULL)";
+
+  g_static_mutex_lock (&__dbg_functions_mutex);
+  if (G_LIKELY (__gst_function_pointers)) {
+    ptrname = g_hash_table_lookup (__gst_function_pointers, (gpointer) func);
     g_static_mutex_unlock (&__dbg_functions_mutex);
-    if (ptrname)
+    if (G_LIKELY (ptrname))
       return ptrname;
+  } else {
+    g_static_mutex_unlock (&__dbg_functions_mutex);
   }
   /* we need to create an entry in the hash table for this one so we don't leak
    * the name */
 #ifdef HAVE_DLADDR
-  if (dladdr (ptr, &dlinfo) && dlinfo.dli_sname) {
+  if (dladdr ((gpointer) func, &dlinfo) && dlinfo.dli_sname) {
     gchar *name = g_strdup (dlinfo.dli_sname);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   } else
 #endif
   {
-    gchar *name = g_strdup_printf ("%p", ptr);
+    gchar *name = g_strdup_printf ("%p", (gpointer) func);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   }
 }
 
 void
-_gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
+_gst_debug_register_funcptr (GstDebugFuncPtr func, const gchar * ptrname)
 {
   gpointer ptr = (gpointer) func;
 
@@ -1157,14 +1258,14 @@ _gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
   if (!__gst_function_pointers)
     __gst_function_pointers = g_hash_table_new (g_direct_hash, g_direct_equal);
   if (!g_hash_table_lookup (__gst_function_pointers, ptr))
-    g_hash_table_insert (__gst_function_pointers, ptr, ptrname);
+    g_hash_table_insert (__gst_function_pointers, ptr, (gpointer) ptrname);
 
   g_static_mutex_unlock (&__dbg_functions_mutex);
 }
 
 #ifdef HAVE_PRINTF_EXTENSION
 static int
-_gst_info_printf_extension (FILE * stream, const struct printf_info *info,
+_gst_info_printf_extension_ptr (FILE * stream, const struct printf_info *info,
     const void *const *args)
 {
   char *buffer;
@@ -1178,7 +1279,26 @@ _gst_info_printf_extension (FILE * stream, const struct printf_info *info,
   len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
       buffer);
 
-  free (buffer);
+  g_free (buffer);
+  return len;
+}
+
+static int
+_gst_info_printf_extension_segment (FILE * stream,
+    const struct printf_info *info, const void *const *args)
+{
+  char *buffer;
+  int len;
+  void *ptr;
+
+  buffer = NULL;
+  ptr = *(void **) args[0];
+
+  buffer = gst_debug_print_segment (ptr);
+  len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
+      buffer);
+
+  g_free (buffer);
   return len;
 }
 
