@@ -26,19 +26,40 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gst/gstatomic.h>
 #include <gst/gstconfig.h>
 
 G_BEGIN_DECLS
 
-/*
- * GStreamer's debugging subsystem is an easy way to get information about what
- * the application is doing.
- * It is not meant for programming errors. Use GLibs methods (g_warning and so
- * on for that.
+/**
+ * GstDebugLevel:
+ * @GST_LEVEL_NONE: No debugging level specified or desired. Used to deactivate
+ *  debugging output.
+ * @GST_LEVEL_ERROR: Error messages are to be used only when an error occured
+ *  that stops the application from keeping working correctly.
+ *  An examples is gst_element_error, which outputs a message with this priority.
+ *  It does not mean that the application is terminating as with g_errror.
+ * @GST_LEVEL_WARNING: Warning messages are to inform about abnormal behaviour
+ *  that could lead to problems or weird behaviour later on. An example of this
+ *  would be clocking issues ("your computer is pretty slow") or broken input
+ *  data ("Can't synchronize to stream.")
+ * @GST_LEVEL_INFO: Informational messages should be used to keep the developer
+ *  updated about what is happening.
+ *  Examples where this should be used are when a typefind function has
+ *  successfully determined the type of the stream or when an mp3 plugin detects
+ *  the format to be used. ("This file has mono sound.")
+ * @GST_LEVEL_DEBUG: Debugging messages should be used when something common
+ *  happens that is not the expected default behavior.
+ *  An example would be notifications about state changes or receiving/sending of
+ *  events.
+ * @GST_LEVEL_LOG: Log messages are messages that are very common but might be
+ *  useful to know. As a rule of thumb a pipeline that is iterating as expected
+ *  should never output anzthing else but LOG messages.
+ *  Examples for this are referencing/dereferencing of objects or cothread switches.
+ * @GST_LEVEL_COUNT: The number of defined debugging levels.
+ *
+ * The level defines the importance of a debugging message. The more important a
+ * message is, the greater the probability that the debugging system outputs it.
  */
-
-/* log levels */
 typedef enum {
   GST_LEVEL_NONE = 0,
   GST_LEVEL_ERROR,
@@ -50,8 +71,16 @@ typedef enum {
   GST_LEVEL_COUNT
 } GstDebugLevel;
 
-/* we can now override this to be more general in maintainer builds
- * or cvs checkouts */
+/**
+ * GST_LEVEL_DEFAULT:
+ *
+ * Defines the default debugging level to be used with GStreamer. It is normally
+ * set to #GST_LEVEL_NONE so nothing get printed.
+ * As it can be configured at compile time, developer builds may chose to
+ * override that though.
+ * You can use this as an argument to gst_debug_set_default_threshold() to
+ * reset the debugging output to default behaviour.
+ */
 #ifndef GST_LEVEL_DEFAULT
 #define GST_LEVEL_DEFAULT GST_LEVEL_NONE
 #endif
@@ -65,6 +94,30 @@ typedef enum {
  * Background color codes:
  * 40=black 41=red 42=green 43=yellow 44=blue 45=magenta 46=cyan 47=white
  */
+/**
+ * GstDebugColorFlags:
+ * @GST_DEBUG_FG_BLACK: Use black as foreground color.
+ * @GST_DEBUG_FG_RED: Use red as foreground color.
+ * @GST_DEBUG_FG_GREEN: Use green as foreground color.
+ * @GST_DEBUG_FG_YELLOW: Use yellow as foreground color.
+ * @GST_DEBUG_FG_BLUE: Use blue as foreground color.
+ * @GST_DEBUG_FG_MAGENTA: Use magenta as foreground color.
+ * @GST_DEBUG_FG_CYAN: Use cyan as foreground color.
+ * @GST_DEBUG_FG_WHITE: Use white as foreground color.
+ * @GST_DEBUG_BG_BLACK: Use black as background color.
+ * @GST_DEBUG_BG_RED: Use red as background color.
+ * @GST_DEBUG_BG_GREEN: Use green as background color.
+ * @GST_DEBUG_BG_YELLOW: Use yellow as background color.
+ * @GST_DEBUG_BG_BLUE: Use blue as background color.
+ * @GST_DEBUG_BG_MAGENTA: Use magenta as background color.
+ * @GST_DEBUG_BG_CYAN: Use cyan as background color.
+ * @GST_DEBUG_BG_WHITE: Use white as background color.
+ * @GST_DEBUG_BOLD: Make the output bold.
+ * @GST_DEBUG_UNDERLINE: Underline the output.
+ *
+ * These are some terminal style flags you can use when creating your
+ * debugging categories to make them stand out in debugging output.
+ */
 typedef enum {
   /* colors */
   GST_DEBUG_FG_BLACK		= 0x0000,
@@ -94,9 +147,15 @@ typedef enum {
 #define GST_DEBUG_FORMAT_MASK	(0xFF00)
 
 typedef struct _GstDebugCategory GstDebugCategory;
+/**
+ * GstDebugCategory:
+ *
+ * This is the struct that describes the categories. Once initialized with
+ * #GST_DEBUG_CATEGORY_INIT, its values can't be changed anymore.
+ */
 struct _GstDebugCategory {
   /*< private >*/
-  GstAtomicInt *	threshold;
+  gint                  threshold;
   guint			color;		/* see defines above */
 
   const gchar *		name;
@@ -105,16 +164,41 @@ struct _GstDebugCategory {
 
 /********** some convenience macros for debugging **********/
 
-/* This is needed in printf's if a char* might be NULL. Solaris crashes then. */
+/**
+ * GST_STR_NULL:
+ * @str: The string to check.
+ *
+ * Macro to use when a string must not be NULL, but may be NULL. If the string
+ * is NULL, "(NULL)" is printed instead.
+ * In GStreamer printf string arguments may not be NULL, because on some
+ * platforms (ie Solaris) the libc crashes in that case. This includes debugging
+ * strings.
+ */
 #define GST_STR_NULL(str) ((str) ? (str) : "(NULL)")
 
-/* easier debugging for pad names */
+/* FIXME, not MT safe */
+/**
+ * GST_DEBUG_PAD_NAME:
+ * @pad: The pad to debug.
+ *
+ * Evaluates to 2 strings, that describe the pad. Often used in debugging
+ * statements.
+ */
 #define GST_DEBUG_PAD_NAME(pad) \
-  (GST_OBJECT_PARENT(pad) != NULL) ? \
+  (pad != NULL) ?  \
+  ((GST_OBJECT_PARENT(pad) != NULL) ? \
   GST_STR_NULL (GST_OBJECT_NAME (GST_OBJECT_PARENT(pad))) : \
-  "''", GST_OBJECT_NAME (pad)
+  "''" ) : "''", \
+  (pad != NULL) ? GST_STR_NULL (GST_OBJECT_NAME (pad)) : "''"
 
-/* You might want to define GST_FUNCTION in apps' configure script. */
+/**
+ * GST_FUNCTION:
+ *
+ * This macro should evaluate to the name of the current function and be should
+ * be defined when configuring your project, as it is compiler dependant. If it
+ * is not defined, some default value is used. It is used to provide debugging
+ * output with the function name of the message.
+ */
 #ifndef GST_FUNCTION
 #if defined (__GNUC__)
 #  define GST_FUNCTION     ((const char*) (__PRETTY_FUNCTION__))
@@ -127,68 +211,100 @@ struct _GstDebugCategory {
 
 
 typedef struct _GstDebugMessage GstDebugMessage;
-typedef void (*GstLogFunction)	(GstDebugCategory *	category,
-				 GstDebugLevel		level,
-				 const gchar *		file,
-				 const gchar *		function,
-				 gint			line,
-				 GObject *		object,
-				 GstDebugMessage *	message,
-				 gpointer		data);
+
+/**
+ * GstLogFunction:
+ * @category: a #GstDebugCategory
+ * @level: a #GstDebugLevel
+ * @file: file name
+ * @function: function name
+ * @line: line number
+ * @object: a #GObject
+ * @message: the message
+ * @data: user data for the log function
+ *
+ * Function prototype for a logging function that can be registered with
+ * gst_debug_add_log_function().
+ * Use G_GNUC_NO_INSTRUMENT on that function.
+ */
+typedef void (*GstLogFunction)  (GstDebugCategory * category,
+                                 GstDebugLevel      level,
+                                 const gchar      * file,
+                                 const gchar      * function,
+                                 gint               line,
+                                 GObject          * object,
+                                 GstDebugMessage  * message,
+                                 gpointer           data);
 
 #ifndef GST_DISABLE_GST_DEBUG
 
-void		_gst_debug_init			(void);
+void            _gst_debug_init (void);
 
-/* note we can't use G_GNUC_PRINTF (7, 8) because gcc chokes on %P, which
- * we use for GST_PTR_FORMAT. */
-void		gst_debug_log		(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 const gchar *		format,
-					 ...) G_GNUC_NO_INSTRUMENT;
-void		gst_debug_log_valist	(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 const gchar *		format,
-					 va_list		args) G_GNUC_NO_INSTRUMENT;
 
-const gchar *	gst_debug_message_get	(GstDebugMessage *	message);
+#ifdef GST_USING_PRINTF_EXTENSION
 
-void		gst_debug_log_default	(GstDebugCategory *	category,
-					 GstDebugLevel		level,
-					 const gchar *		file,
-					 const gchar *		function,
-					 gint			line,
-					 GObject *		object,
-					 GstDebugMessage *	message,
-					 gpointer		unused) G_GNUC_NO_INSTRUMENT;
+/* not using G_GNUC_PRINTF, since gcc will choke on GST_PTR_FORMAT being %P */
+void		    gst_debug_log            (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          ...) G_GNUC_NO_INSTRUMENT;
+
+#else /* GST_USING_PRINTF_EXTENSION */
+
+void		    gst_debug_log            (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          ...) G_GNUC_PRINTF (7, 8) G_GNUC_NO_INSTRUMENT;
+
+#endif /* GST_USING_PRINTF_EXTENSION */
+
+void            gst_debug_log_valist     (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint	              line,
+                                          GObject          * object,
+                                          const gchar      * format,
+                                          va_list            args) G_GNUC_NO_INSTRUMENT;
+
+const gchar   * gst_debug_message_get    (GstDebugMessage  * message);
+
+void            gst_debug_log_default    (GstDebugCategory * category,
+                                          GstDebugLevel      level,
+                                          const gchar      * file,
+                                          const gchar      * function,
+                                          gint               line,
+                                          GObject          * object,
+                                          GstDebugMessage  * message,
+                                          gpointer           unused) G_GNUC_NO_INSTRUMENT;
 
 G_CONST_RETURN gchar *
-		gst_debug_level_get_name	(GstDebugLevel		level);
+                gst_debug_level_get_name (GstDebugLevel level);
 
-void		gst_debug_add_log_function	(GstLogFunction		func,
-						 gpointer data);
-guint		gst_debug_remove_log_function	(GstLogFunction		func);
-guint		gst_debug_remove_log_function_by_data (gpointer		data);
+void            gst_debug_add_log_function            (GstLogFunction func,
+                                                       gpointer       data);
+guint           gst_debug_remove_log_function         (GstLogFunction func);
+guint           gst_debug_remove_log_function_by_data (gpointer       data);
 
-void		gst_debug_set_active		(gboolean active);
-gboolean	gst_debug_is_active		(void);
+void            gst_debug_set_active  (gboolean active);
+gboolean        gst_debug_is_active   (void);
 
-void		gst_debug_set_colored		(gboolean colored);
-gboolean	gst_debug_is_colored		(void);
+void            gst_debug_set_colored (gboolean colored);
+gboolean        gst_debug_is_colored  (void);
 
-void		gst_debug_set_default_threshold	(GstDebugLevel		level);
-GstDebugLevel	gst_debug_get_default_threshold	(void);
-void		gst_debug_set_threshold_for_name (const gchar *		name,
-						 GstDebugLevel		level);
-void		gst_debug_unset_threshold_for_name (const gchar *	name);
+void            gst_debug_set_default_threshold	    (GstDebugLevel level);
+GstDebugLevel   gst_debug_get_default_threshold	    (void);
+void            gst_debug_set_threshold_for_name    (const gchar * name,
+                                                     GstDebugLevel level);
+void            gst_debug_unset_threshold_for_name  (const gchar * name);
 
 /**
  * GST_DEBUG_CATEGORY:
@@ -205,7 +321,13 @@ void		gst_debug_unset_threshold_for_name (const gchar *	name);
  * Declares a GstDebugCategory variable as extern. Use in header files.
  * This macro expands to nothing if debugging is disabled.
  */
+#ifndef _MSC_VER
 #define GST_DEBUG_CATEGORY_EXTERN(cat) extern GstDebugCategory *cat
+#else /* _MSC_VER */
+#define GST_DEBUG_CATEGORY_EXTERN(cat) \
+  extern __declspec (dllimport) GstDebugCategory *cat;
+#endif
+
 /**
  * GST_DEBUG_CATEGORY_STATIC:
  * @cat: the category
@@ -275,13 +397,33 @@ GSList *
 gchar *	gst_debug_construct_term_color	(guint colorinfo);
 
 
+/**
+ * GST_CAT_DEFAULT:
+ *
+ * Default gstreamer core debug log category. Please define your own.
+ */
 GST_EXPORT GstDebugCategory *	GST_CAT_DEFAULT;
 /* this symbol may not be used */
 extern gboolean			__gst_debug_enabled;
 
+/* since 0.10.7, the min debug level, used for quickly discarding debug
+ * messages that fall under the threshold. */
+extern GstDebugLevel            __gst_debug_min; 
+
+/**
+ * GST_CAT_LEVEL_LOG:
+ * @cat: category to use
+ * @level: the severity of the message
+ * @object: the #GObject the message belongs to or NULL if none
+ * @...: A printf-style message to output
+ *
+ * Outputs a debugging message. This is the most general macro for outputting
+ * debugging messages. You will probably want to use one of the ones described
+ * below.
+ */
 #ifdef G_HAVE_ISO_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,...) G_STMT_START{		\
-  if (__gst_debug_enabled) {						\
+  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), __VA_ARGS__);				\
   }									\
@@ -289,7 +431,7 @@ extern gboolean			__gst_debug_enabled;
 #else /* G_HAVE_GNUC_VARARGS */
 #ifdef G_HAVE_GNUC_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,args...) G_STMT_START{	\
-  if (__gst_debug_enabled) {						\
+  if (G_UNLIKELY (level <= __gst_debug_min)) {						\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), ##args );					\
   }									\
@@ -307,7 +449,7 @@ static inline void
 GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
     gpointer object, const char *format, ...)
 {
-  if (__gst_debug_enabled) {
+  if (G_UNLIKELY (level <= __gst_debug_min)) {
     va_list varargs;
 
     va_start (varargs, format);
@@ -318,6 +460,156 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
 #endif
 #endif /* G_HAVE_ISO_VARARGS */
 
+/**
+ * GST_CAT_ERROR_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an error message belonging to the given object in the given category.
+ */
+/**
+ * GST_CAT_WARNING_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a warning message belonging to the given object in the given category.
+ */
+/**
+ * GST_CAT_INFO_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an informational message belonging to the given object in the given
+ * category.
+ */
+/**
+ * GST_CAT_DEBUG_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an debugging message belonging to the given object in the given category.
+ */
+/**
+ * GST_CAT_LOG_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an logging message belonging to the given object in the given category.
+ */
+
+
+/**
+ * GST_CAT_ERROR:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an error message in the given category.
+ */
+/**
+ * GST_CAT_WARNING:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an warning message in the given category.
+ */
+/**
+ * GST_CAT_INFO:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an informational message in the given category.
+ */
+/**
+ * GST_CAT_DEBUG:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an debugging message in the given category.
+ */
+/**
+ * GST_CAT_LOG:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an logging message in the given category.
+ */
+
+
+/**
+ * GST_ERROR_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an error message belonging to the given object in the default category.
+ */
+/**
+ * GST_WARNING_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a warning message belonging to the given object in the default category.
+ */
+/**
+ * GST_INFO_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output an informational message belonging to the given object in the default
+ * category.
+ */
+/**
+ * GST_DEBUG_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a debugging message belonging to the given object in the default
+ * category.
+ */
+/**
+ * GST_LOG_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a logging message belonging to the given object in the default category.
+ */
+
+
+/**
+ * GST_ERROR:
+ * @...: printf-style message to output
+ *
+ * Output an error message in the default category.
+ */
+/**
+ * GST_WARNING:
+ * @...: printf-style message to output
+ *
+ * Output a warning message in the default category.
+ */
+/**
+ * GST_INFO:
+ * @...: printf-style message to output
+ *
+ * Output an informational message in the default category.
+ */
+/**
+ * GST_DEBUG:
+ * @...: printf-style message to output
+ *
+ * Output a debugging message in the default category.
+ */
+/**
+ * GST_LOG:
+ * @...: printf-style message to output
+ *
+ * Output a logging message in the default category.
+ */
+
 #ifdef G_HAVE_ISO_VARARGS
 
 #define GST_CAT_ERROR_OBJECT(cat,obj,...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_ERROR,   obj,  __VA_ARGS__)
@@ -592,13 +884,35 @@ GST_LOG (const char *format, ...)
 
 
 /********** function pointer stuff **********/
-void *	_gst_debug_register_funcptr	(void *			ptr,
-					 gchar *		ptrname);
-G_CONST_RETURN gchar *
-	_gst_debug_nameof_funcptr	(void *			ptr);
 
-#define GST_DEBUG_FUNCPTR(ptr) (_gst_debug_register_funcptr((void *)(ptr), #ptr) , ptr)
-#define GST_DEBUG_FUNCPTR_NAME(ptr) _gst_debug_nameof_funcptr((void *)ptr)
+typedef	void (* GstDebugFuncPtr)	(void);
+void	_gst_debug_register_funcptr	(GstDebugFuncPtr	func,
+					 const gchar *		ptrname);
+G_CONST_RETURN gchar *
+	_gst_debug_nameof_funcptr	(GstDebugFuncPtr	func);
+
+/**
+ * GST_DEBUG_FUNCPTR:
+ * @ptr: pointer to the function to register
+ *
+ * Register a pointer to a function with its name, so it can later be used by
+ * GST_DEBUG_FUNCPTR_NAME().
+ */
+#define GST_DEBUG_FUNCPTR(ptr) \
+  (_gst_debug_register_funcptr((GstDebugFuncPtr)(ptr), #ptr) , ptr)
+
+/**
+ * GST_DEBUG_FUNCPTR_NAME:
+ * @ptr: address of the function of which to look up the name
+ *
+ * Retrieves the name of the function, if it was previously registered with
+ * GST_DEBUG_FUNCPTR(). If not, it returns a description of the pointer.
+ *
+ * This macro returns a constant string which must not be modified or
+ * freed by the caller.
+ */
+#define GST_DEBUG_FUNCPTR_NAME(ptr) \
+  _gst_debug_nameof_funcptr((GstDebugFuncPtr)ptr)
 
 #else /* GST_DISABLE_GST_DEBUG */
 
@@ -610,15 +924,14 @@ G_CONST_RETURN gchar *
 
 #define _gst_debug_init()				/* NOP */
 
-#define gst_debug_set_log_function(func,data)		/* NOP */
-#define gst_debug_reset_log_function(void)		/* NOP */
 #define gst_debug_set_default_threshold(level)		/* NOP */
 #define gst_debug_get_default_threshold()		(GST_LEVEL_NONE)
 
-#define gst_debug_level_get_name(level)			("NONE")
-#define gst_debug_add_log_function(func,data)		(FALSE)
-#define gst_debug_remove_log_function(func)		(0)
-#define gst_debug_remove_log_function_by_data(data)	(0)
+#define gst_debug_level_get_name(level)				("NONE")
+#define gst_debug_message_get(message)  			("NONE")
+#define gst_debug_add_log_function(func,data)		/* NOP */
+guint gst_debug_remove_log_function (GstLogFunction func);
+guint gst_debug_remove_log_function_by_data (gpointer data);
 #define gst_debug_set_active(active)			/* NOP */
 #define gst_debug_is_active()				(FALSE)
 #define gst_debug_set_colored(colored)			/* NOP */
@@ -824,15 +1137,6 @@ GST_LOG (const char *format, ...)
 
 void gst_debug_print_stack_trace (void);
 
-/* timestamp debugging macros */
-/* FIXME 0.9: move into the correct header (gstclock.h) */
-#define GST_TIME_FORMAT "u:%02u:%02u.%09u"
-#define GST_TIME_ARGS(t) \
-	(guint) ((t) / (GST_SECOND * 60 * 60)), \
-	(guint) (((t) / (GST_SECOND * 60)) % 60), \
-	(guint) (((t) / GST_SECOND) % 60), \
-	(guint) ((t) % GST_SECOND)
-
 G_END_DECLS
 
 #endif /* __GSTINFO_H__ */
