@@ -25,8 +25,6 @@
 #ifndef __GST_UTILS_H__
 #define __GST_UTILS_H__
 
-#include <gst/gstconfig.h>
-
 #include <glib.h>
 #include <gst/gstbin.h>
 
@@ -72,6 +70,21 @@ void		gst_print_pad_caps		(GString *buf, gint indent, GstPad *pad);
 void		gst_print_element_args		(GString *buf, gint indent, GstElement *element);
 
 
+GType gst_type_register_static_full (GType parent_type,
+			                   const gchar       *type_name,
+                               guint              class_size,
+                               GBaseInitFunc      base_init,
+                               GBaseFinalizeFunc  base_finalize,
+                               GClassInitFunc     class_init,
+                               GClassFinalizeFunc class_finalize,
+                               gconstpointer      class_data,
+                               guint              instance_size,
+                               guint16            n_preallocs,
+                               GInstanceInitFunc  instance_init,
+                               const GTypeValueTable *value_table,
+                               GTypeFlags	 flags);
+
+
 /* Macros for defining classes.  Ideas taken from Bonobo, which took theirs
    from Nautilus and GOB. */
 
@@ -117,19 +130,18 @@ type_as_function ## _get_type (void)					\
 {									\
   static GType object_type = 0;						\
   if (G_UNLIKELY (object_type == 0)) {					\
-    static const GTypeInfo object_info = {				\
-      sizeof (type ## Class),						\
-      type_as_function ## _base_init,					\
-      NULL,		  /* base_finalize */				\
-      type_as_function ## _class_init_trampoline,			\
-      NULL,		  /* class_finalize */				\
-      NULL,               /* class_data */				\
-      sizeof (type),							\
-      0,                  /* n_preallocs */				\
-      (GInstanceInitFunc) type_as_function ## _init			\
-    };									\
-    object_type = g_type_register_static (parent_type_macro, #type,	\
-	&object_info, (GTypeFlags) 0);					\
+    object_type = gst_type_register_static_full (parent_type_macro, #type,	\
+	sizeof (type ## Class),					\
+        type_as_function ## _base_init,					\
+        NULL,		  /* base_finalize */				\
+        type_as_function ## _class_init_trampoline,			\
+        NULL,		  /* class_finalize */				\
+        NULL,               /* class_data */				\
+        sizeof (type),							\
+        0,                  /* n_preallocs */				\
+        (GInstanceInitFunc) type_as_function ## _init,                  \
+        NULL,                                                           \
+        (GTypeFlags) 0);				                \
     additional_initializations (object_type);				\
   }									\
   return object_type;							\
@@ -462,47 +474,102 @@ GST_BOILERPLATE_FULL (type, type_as_function, parent_type,              \
 
 /**
  * GST_ROUND_UP_2:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by two without a rest.
+ * Rounds an integer value up to the next multiple of 2.
  */
 #define GST_ROUND_UP_2(num)  (((num)+1)&~1)
 /**
  * GST_ROUND_UP_4:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by four without a rest.
+ * Rounds an integer value up to the next multiple of 4.
  */
 #define GST_ROUND_UP_4(num)  (((num)+3)&~3)
 /**
  * GST_ROUND_UP_8:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by eight without a rest.
+ * Rounds an integer value up to the next multiple of 8.
  */
 #define GST_ROUND_UP_8(num)  (((num)+7)&~7)
 /**
  * GST_ROUND_UP_16:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by 16 without a rest.
+ * Rounds an integer value up to the next multiple of 16.
  */
 #define GST_ROUND_UP_16(num) (((num)+15)&~15)
 /**
  * GST_ROUND_UP_32:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by 32 without a rest.
+ * Rounds an integer value up to the next multiple of 32.
  */
 #define GST_ROUND_UP_32(num) (((num)+31)&~31)
 /**
  * GST_ROUND_UP_64:
- * @num: value round up
+ * @num: integer value to round up
  *
- * Make number divideable by 64 without a rest.
+ * Rounds an integer value up to the next multiple of 64.
  */
 #define GST_ROUND_UP_64(num) (((num)+63)&~63)
 
+/**
+ * GST_ROUND_DOWN_2:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 2.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_2(num)  ((num)&(~1))
+/**
+ * GST_ROUND_DOWN_4:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 4.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_4(num)  ((num)&(~3))
+/**
+ * GST_ROUND_DOWN_8:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 8.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_8(num)  ((num)&(~7))
+/**
+ * GST_ROUND_DOWN_16:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 16.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_16(num) ((num)&(~15))
+/**
+ * GST_ROUND_DOWN_32:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 32.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_32(num) ((num)&(~31))
+/**
+ * GST_ROUND_DOWN_64:
+ * @num: integer value to round down
+ *
+ * Rounds an integer value down to the next multiple of 64.
+ *
+ * Since: 0.10.12
+ */
+#define GST_ROUND_DOWN_64(num) ((num)&(~63))
+
 void			gst_object_default_error	(GstObject * source,
 							 GError * error, gchar * debug);
 
@@ -514,6 +581,7 @@ GstPad*                 gst_element_get_compatible_pad  (GstElement *element, Gs
 GstPadTemplate*         gst_element_get_compatible_pad_template (GstElement *element, GstPadTemplate *compattempl);
 
 G_CONST_RETURN gchar*   gst_element_state_get_name      (GstState state);
+G_CONST_RETURN gchar *  gst_element_state_change_return_get_name (GstStateChangeReturn state_ret);
 
 gboolean		gst_element_link                (GstElement *src, GstElement *dest);
 gboolean		gst_element_link_many           (GstElement *element_1,
@@ -588,7 +656,9 @@ GstPad *                gst_bin_find_unconnected_pad    (GstBin *bin, GstPadDire
 /* buffer functions */
 GstBuffer *		gst_buffer_merge		(GstBuffer * buf1, GstBuffer * buf2);
 GstBuffer *		gst_buffer_join			(GstBuffer * buf1, GstBuffer * buf2);
+#ifndef GST_DISABLE_DEPRECATED
 void			gst_buffer_stamp		(GstBuffer * dest, const GstBuffer * src);
+#endif /* GST_DISABLE_DEPRECATED */
 
 /* atomic functions */
 void                    gst_atomic_int_set              (gint * atomic_int, gint value);
@@ -614,18 +684,10 @@ void			gst_element_found_tags_for_pad	(GstElement * element,
 void			gst_element_found_tags		(GstElement * element,
 							 GstTagList * list);
 
-#ifndef GST_DISABLE_PARSE
 /* parse utility functions */
 GstElement *            gst_parse_bin_from_description  (const gchar * bin_description,
                                                          gboolean ghost_unconnected_pads,
                                                          GError ** err);
-#else /* GST_DISABLE_PARSE */
-
-#if defined _GNUC_ && _GNUC_ >= 3
-#pragma GCC poison gst_parse_bin_from_description 
-#endif
-
-#endif
 
 G_END_DECLS
 
