@@ -20,7 +20,13 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+/**
+ * SECTION:element-identity
+ * @short_description: pass data through without modification
+ *
+ * Dummy element that passes incomming data through unmodified. I has some
+ * useful diagnostic functions, such as offset and timestamp checking.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -45,13 +51,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_identity_debug);
 #define GST_CAT_DEFAULT gst_identity_debug
 
-static const GstElementDetails gst_identity_details =
-GST_ELEMENT_DETAILS ("Identity",
-    "Generic",
-    "Pass data without modification",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
-
 /* Identity signals and args */
 enum
 {
@@ -70,6 +69,8 @@ enum
 #define DEFAULT_DUMP                    FALSE
 #define DEFAULT_SYNC                    FALSE
 #define DEFAULT_CHECK_PERFECT           FALSE
+#define DEFAULT_CHECK_IMPERFECT_TIMESTAMP FALSE
+#define DEFAULT_CHECK_IMPERFECT_OFFSET    FALSE
 
 enum
 {
@@ -83,7 +84,9 @@ enum
   PROP_LAST_MESSAGE,
   PROP_DUMP,
   PROP_SYNC,
-  PROP_CHECK_PERFECT
+  PROP_CHECK_PERFECT,
+  PROP_CHECK_IMPERFECT_TIMESTAMP,
+  PROP_CHECK_IMPERFECT_OFFSET
 };
 
 
@@ -112,11 +115,14 @@ gst_identity_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "Identity",
+      "Generic",
+      "Pass data without modification", "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&sinktemplate));
-  gst_element_class_set_details (gstelement_class, &gst_identity_details);
 }
 
 static void
@@ -183,7 +189,7 @@ gst_identity_class_init (GstIdentityClass * klass)
   g_object_class_install_property (gobject_class,
       PROP_DROP_PROBABILITY, g_param_spec_float ("drop_probability",
           "Drop Probability",
-          "The Probability a buffer is dropped (not implemented)", 0.0, 1.0,
+          "The Probability a buffer is dropped", 0.0, 1.0,
           DEFAULT_DROP_PROBABILITY, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_DATARATE,
       g_param_spec_int ("datarate", "Datarate",
@@ -207,8 +213,21 @@ gst_identity_class_init (GstIdentityClass * klass)
           "Synchronize to pipeline clock", DEFAULT_SYNC, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_CHECK_PERFECT,
       g_param_spec_boolean ("check-perfect", "Check For Perfect Stream",
-          "Verify that the stream is time- and data-contiguous",
+          "Verify that the stream is time- and data-contiguous. "
+          "This only logs in the debug log.  This will be deprecated in favor "
+          "of the check-imperfect-timestamp/offset properties.",
           DEFAULT_CHECK_PERFECT, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class,
+      PROP_CHECK_IMPERFECT_TIMESTAMP,
+      g_param_spec_boolean ("check-imperfect-timestamp",
+          "Check for discontiguous timestamps",
+          "Send element messages if timestamps and durations do not match up",
+          DEFAULT_CHECK_IMPERFECT_TIMESTAMP, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_CHECK_IMPERFECT_OFFSET,
+      g_param_spec_boolean ("check-imperfect-offset",
+          "Check for discontiguous offset",
+          "Send element messages if offset and offset_end do not match up",
+          DEFAULT_CHECK_IMPERFECT_OFFSET, G_PARAM_READWRITE));
 
    /**
    * GstIdentity::handoff:
@@ -245,6 +264,8 @@ gst_identity_init (GstIdentity * identity, GstIdentityClass * g_class)
   identity->single_segment = DEFAULT_SINGLE_SEGMENT;
   identity->sync = DEFAULT_SYNC;
   identity->check_perfect = DEFAULT_CHECK_PERFECT;
+  identity->check_imperfect_timestamp = DEFAULT_CHECK_IMPERFECT_TIMESTAMP;
+  identity->check_imperfect_offset = DEFAULT_CHECK_IMPERFECT_OFFSET;
   identity->dump = DEFAULT_DUMP;
   identity->last_message = NULL;
 }
@@ -318,15 +339,22 @@ gst_identity_check_perfect (GstIdentity * identity, GstBuffer * buf)
   /* invalid timestamp drops us out of check.  FIXME: maybe warn ? */
   if (timestamp != GST_CLOCK_TIME_NONE) {
     /* check if we had a previous buffer to compare to */
-    if (identity->prev_timestamp != GST_CLOCK_TIME_NONE) {
-      guint64 offset;
+    if (identity->prev_timestamp != GST_CLOCK_TIME_NONE &&
+        identity->prev_duration != GST_CLOCK_TIME_NONE) {
+      guint64 offset, t_expected;
+      gint64 dt;
 
-      if (identity->prev_timestamp + identity->prev_duration != timestamp) {
+      t_expected = identity->prev_timestamp + identity->prev_duration;
+      dt = timestamp - t_expected;
+      if (dt != 0) {
         GST_WARNING_OBJECT (identity,
             "Buffer not time-contiguous with previous one: " "prev ts %"
             GST_TIME_FORMAT ", prev dur %" GST_TIME_FORMAT ", new ts %"
-            GST_TIME_FORMAT, GST_TIME_ARGS (identity->prev_timestamp),
-            GST_TIME_ARGS (identity->prev_duration), GST_TIME_ARGS (timestamp));
+            GST_TIME_FORMAT " (expected ts %" GST_TIME_FORMAT ", delta=%c%"
+            GST_TIME_FORMAT ")", GST_TIME_ARGS (identity->prev_timestamp),
+            GST_TIME_ARGS (identity->prev_duration), GST_TIME_ARGS (timestamp),
+            GST_TIME_ARGS (t_expected), (dt < 0) ? '-' : '+',
+            GST_TIME_ARGS ((dt < 0) ? (GstClockTime) (-dt) : dt));
       }
 
       offset = GST_BUFFER_OFFSET (buf);
@@ -336,11 +364,107 @@ gst_identity_check_perfect (GstIdentity * identity, GstBuffer * buf)
             "prev offset_end %" G_GINT64_FORMAT ", new offset %"
             G_GINT64_FORMAT, identity->prev_offset_end, offset);
       }
+    } else {
+      GST_DEBUG_OBJECT (identity, "can't check time-contiguity, no timestamp "
+          "and/or duration were set on previous buffer");
     }
-    /* update prev values */
-    identity->prev_timestamp = timestamp;
-    identity->prev_duration = GST_BUFFER_DURATION (buf);
-    identity->prev_offset_end = GST_BUFFER_OFFSET_END (buf);
+  }
+}
+
+static void
+gst_identity_check_imperfect_timestamp (GstIdentity * identity, GstBuffer * buf)
+{
+  GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buf);
+
+  /* invalid timestamp drops us out of check.  FIXME: maybe warn ? */
+  if (timestamp != GST_CLOCK_TIME_NONE) {
+    /* check if we had a previous buffer to compare to */
+    if (identity->prev_timestamp != GST_CLOCK_TIME_NONE &&
+        identity->prev_duration != GST_CLOCK_TIME_NONE) {
+      guint64 offset;
+      GstClockTime t_expected;
+      GstClockTimeDiff dt;
+
+      offset = GST_BUFFER_OFFSET (buf);
+      t_expected = identity->prev_timestamp + identity->prev_duration;
+      dt = GST_CLOCK_DIFF (t_expected, timestamp);
+      if (dt != 0) {
+        /*
+         * "imperfect-timestamp" bus message:
+         * @identity:        the identity instance
+         * @prev-timestamp:  the previous buffer timestamp
+         * @prev-duration:   the previous buffer duration
+         * @prev-offset:     the previous buffer offset
+         * @prev-offset-end: the previous buffer offset end
+         * @cur-timestamp:   the current buffer timestamp
+         * @cur-duration:    the current buffer duration
+         * @cur-offset:      the current buffer offset
+         * @cur-offset-end:  the current buffer offset end
+         *
+         * This bus message gets emitted if the check-imperfect-timestamp
+         * property is set and there is a gap in time between the
+         * last buffer and the newly received buffer.
+         */
+        gst_element_post_message (GST_ELEMENT (identity),
+            gst_message_new_element (GST_OBJECT (identity),
+                gst_structure_new ("imperfect-timestamp",
+                    "prev-timestamp", G_TYPE_UINT64,
+                    identity->prev_timestamp, "prev-duration", G_TYPE_UINT64,
+                    identity->prev_duration, "prev-offset", G_TYPE_UINT64,
+                    identity->prev_offset, "prev-offset-end", G_TYPE_UINT64,
+                    identity->prev_offset_end, "cur-timestamp", G_TYPE_UINT64,
+                    timestamp, "cur-duration", G_TYPE_UINT64,
+                    GST_BUFFER_DURATION (buf), "cur-offset", G_TYPE_UINT64,
+                    GST_BUFFER_OFFSET (buf), "cur-offset-end", G_TYPE_UINT64,
+                    GST_BUFFER_OFFSET_END (buf), NULL)));
+      }
+    } else {
+      GST_DEBUG_OBJECT (identity, "can't check data-contiguity, no "
+          "offset_end was set on previous buffer");
+    }
+  }
+}
+
+static void
+gst_identity_check_imperfect_offset (GstIdentity * identity, GstBuffer * buf)
+{
+  guint64 offset;
+
+  offset = GST_BUFFER_OFFSET (buf);
+
+  if (identity->prev_offset_end != offset &&
+      identity->prev_offset_end != GST_BUFFER_OFFSET_NONE &&
+      offset != GST_BUFFER_OFFSET_NONE) {
+    /*
+     * "imperfect-offset" bus message:
+     * @identity:        the identity instance
+     * @prev-timestamp:  the previous buffer timestamp
+     * @prev-duration:   the previous buffer duration
+     * @prev-offset:     the previous buffer offset
+     * @prev-offset-end: the previous buffer offset end
+     * @cur-timestamp:   the current buffer timestamp
+     * @cur-duration:    the current buffer duration
+     * @cur-offset:      the current buffer offset
+     * @cur-offset-end:  the current buffer offset end
+     *
+     * This bus message gets emitted if the check-imperfect-offset
+     * property is set and there is a gap in offsets between the
+     * last buffer and the newly received buffer.
+     */
+    gst_element_post_message (GST_ELEMENT (identity),
+        gst_message_new_element (GST_OBJECT (identity),
+            gst_structure_new ("imperfect-offset", "prev-timestamp",
+                G_TYPE_UINT64, identity->prev_timestamp, "prev-duration",
+                G_TYPE_UINT64, identity->prev_duration, "prev-offset",
+                G_TYPE_UINT64, identity->prev_offset, "prev-offset-end",
+                G_TYPE_UINT64, identity->prev_offset_end, "cur-timestamp",
+                G_TYPE_UINT64, GST_BUFFER_TIMESTAMP (buf), "cur-duration",
+                G_TYPE_UINT64, GST_BUFFER_DURATION (buf), "cur-offset",
+                G_TYPE_UINT64, GST_BUFFER_OFFSET (buf), "cur-offset-end",
+                G_TYPE_UINT64, GST_BUFFER_OFFSET_END (buf), NULL)));
+  } else {
+    GST_DEBUG_OBJECT (identity, "can't check offset contiguity, no offset "
+        "and/or offset_end were set on previous buffer");
   }
 }
 
@@ -353,6 +477,16 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
 
   if (identity->check_perfect)
     gst_identity_check_perfect (identity, buf);
+  if (identity->check_imperfect_timestamp)
+    gst_identity_check_imperfect_timestamp (identity, buf);
+  if (identity->check_imperfect_offset)
+    gst_identity_check_imperfect_offset (identity, buf);
+
+  /* update prev values */
+  identity->prev_timestamp = GST_BUFFER_TIMESTAMP (buf);
+  identity->prev_duration = GST_BUFFER_DURATION (buf);
+  identity->prev_offset_end = GST_BUFFER_OFFSET_END (buf);
+  identity->prev_offset = GST_BUFFER_OFFSET (buf);
 
   if (identity->error_after >= 0) {
     identity->error_after--;
@@ -380,10 +514,8 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
         GST_OBJECT_UNLOCK (identity);
         g_object_notify (G_OBJECT (identity), "last-message");
       }
-      /* FIXME, this does not drop the buffer in basetransform. Actually
-       * dropping the buffer in transform_ip is not possible without a new
-       * custom GstFlowReturn value. */
-      return GST_FLOW_OK;
+      /* return DROPPED to basetransform. */
+      return GST_BASE_TRANSFORM_FLOW_DROPPED;
     }
   }
 
@@ -408,7 +540,8 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
   }
 
   if (identity->datarate > 0) {
-    GstClockTime time = identity->offset * GST_SECOND / identity->datarate;
+    GstClockTime time = gst_util_uint64_scale_int (identity->offset,
+        GST_SECOND, identity->datarate);
 
     GST_BUFFER_TIMESTAMP (buf) = time;
     GST_BUFFER_DURATION (buf) =
@@ -501,6 +634,12 @@ gst_identity_set_property (GObject * object, guint prop_id,
     case PROP_CHECK_PERFECT:
       identity->check_perfect = g_value_get_boolean (value);
       break;
+    case PROP_CHECK_IMPERFECT_TIMESTAMP:
+      identity->check_imperfect_timestamp = g_value_get_boolean (value);
+      break;
+    case PROP_CHECK_IMPERFECT_OFFSET:
+      identity->check_imperfect_offset = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -548,6 +687,12 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_CHECK_PERFECT:
       g_value_set_boolean (value, identity->check_perfect);
       break;
+    case PROP_CHECK_IMPERFECT_TIMESTAMP:
+      g_value_set_boolean (value, identity->check_imperfect_timestamp);
+      break;
+    case PROP_CHECK_IMPERFECT_OFFSET:
+      g_value_set_boolean (value, identity->check_imperfect_offset);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -565,6 +710,7 @@ gst_identity_start (GstBaseTransform * trans)
   identity->prev_timestamp = GST_CLOCK_TIME_NONE;
   identity->prev_duration = GST_CLOCK_TIME_NONE;
   identity->prev_offset_end = -1;
+  identity->prev_offset = -1;
 
   return TRUE;
 }
